@@ -182,24 +182,26 @@ sub weakfield {
 	my $name = shift;
 	my $initcode;
 	my $init;
+	my $mode = 'defined';
 	my $assign = '
 		my $val = shift;
 		$self->{$name} = $val;
-		Scalar::Util::weaken($self->{$name}) if ref $val;
+		Scalar::Util::weaken($self->{$name}) if Scalar::Util::reftype($val);
 		return $val;
 	';
 	if(@_) {
 		$init = shift;
-		if(ref $init) {
+		if(Scalar::Util::reftype($init)) {
 			if(Scalar::Util::reftype($init) eq 'CODE') {
-				$initcode = '
+				$initcode = <<'EOT' . $assign;
 					unless(@_ = $init->($self, $name, $package)) {
 						return unless exists $self->{$name};
+						my $val = $self->{$name};
 						Scalar::Util::weaken($self->{$name})
-							if ref $self->{$name};
-						return $self->{$name};
+							if Scalar::Util::reftype($val);
+						return $val;
 					}
-				'.$assign;
+EOT
 			} else {
 				$init = Class::Clarity::Factory::factory($init);
 				$initcode = '@_ = $init->();'.$assign;
@@ -209,14 +211,17 @@ sub weakfield {
 		}
 	} else {
 		$init = "field $name in package $package used uninitialized";
-		$initcode = "Carp::confess(\$init);\n";
+		$initcode = << 'EOT';
+	Carp::confess($init) unless exists $self->{$name};
+EOT
+		$mode = 'exists';
 	}
 	my $code = "
 		package $package;
 		sub $name {
 			my \$self = shift;
 			if(\@_) { $assign }
-			return \$self->{\$name} if exists \$self->{\$name};
+			return \$self->{\$name} if $mode \$self->{\$name};
 			$initcode
 		}
 		sub ${name}_isset { exists shift->{\$name} }
@@ -669,7 +674,7 @@ sub import {
 	}
 
 	local $Exporter::ExportLevel = 1;
-	return &Exporter::export;
+	return &Exporter::import;
 }
 
 sub new {
@@ -826,7 +831,8 @@ You can trigger the default by deleting the field from $self:
 
 You can freely intermix these calls with direct accesses to the fields:
 
-     field name;
+ field name;
+
      @$self{qw(name color quest)} = qw(Bob red fame);
      print $self->name;                 # will print 'Bob'
 
@@ -877,10 +883,12 @@ Creates a simple field, as described above.
 
 Creates a weak field, which is like field() but with the additional feature
 that Scalar::Util::weaken() is called every time the field is set.
+Unlike a normal field, the initializer is also invoked if the field is
+existent but undef.
 
 =head4 const()
 
-A field created using const() will gleefully ignore any attempts to set it.
+A field created using const() will die() if you make any attempts to set it.
 There's exactly only exception: when no default is provided and the field
 is still uninitialized it is allowed to set it once.
 
